about summary refs log tree commit diff
path: root/login/programs
diff options
context:
space:
mode:
Diffstat (limited to 'login/programs')
-rw-r--r--login/programs/database.c99
-rw-r--r--login/programs/request.c6
-rw-r--r--login/programs/utmpd.c19
3 files changed, 82 insertions, 42 deletions
diff --git a/login/programs/request.c b/login/programs/request.c
index 5e6bfa19cf..889ce0cba9 100644
--- a/login/programs/request.c
+++ b/login/programs/request.c
@@ -88,7 +88,7 @@ read_data (client_connection *connection)
     }
 
   if (nbytes < 0)
-    error (0, errno, "cannot read from client");
+    error (0, errno, _("cannot read from client"));
 
   return -1;
 }
@@ -117,7 +117,7 @@ write_data (client_connection *connection)
     }
 
   if (nbytes < 0)
-    error (0, errno, "cannot write to client");
+    error (0, errno, _("cannot write to client"));
 
   return -1;
 }
@@ -164,7 +164,7 @@ send_reply (client_connection *connection, const reply_header *reply)
   /* Check if the reply fits in the buffer.  */
   if ((size_t) (connection->write_end - connection->write_ptr) < reply->size)
     {
-      error (0, 0, "buffer overflow");
+      error (0, 0, _("buffer overflow"));
       return -1;
     }
 
diff --git a/login/programs/utmpd.c b/login/programs/utmpd.c
index 3c8d626a84..2fef776a69 100644
--- a/login/programs/utmpd.c
+++ b/login/programs/utmpd.c
@@ -139,12 +139,12 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
   
   /* Check if we are already running.  */
   if (check_pid (_PATH_UTMPDPID))
-    error (EXIT_FAILURE, 0, "already running");
+    error (EXIT_FAILURE, 0, _("already running"));
 
   /* Open UTMP database.  */
   utmp_db = open_database (_PATH_UTMP "x", _PATH_UTMP);
   if (utmp_db == NULL)
-    error (EXIT_FAILURE, errno, "%s", _PATH_UTMP);
+    exit (EXIT_FAILURE);
 
   /* Create sockets, with the right permissions.  */
   mask = umask (S_IXUSR | S_IXGRP | S_IXOTH);
@@ -156,7 +156,8 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
   /* Set the sockets up to accept connections.  */
   if (listen (ro_sock, MAX_CONNECTIONS) < 0
       || listen (rw_sock, MAX_CONNECTIONS) < 0)
-    error (EXIT_FAILURE, errno, "cannot enable socket to accept connections");
+    error (EXIT_FAILURE, errno,
+	   _("cannot enable socket to accept connections"));
 
   /* Behave like a daemon.  */
   if (!debug)
@@ -164,7 +165,7 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
       openlog ("utmpd", LOG_CONS | LOG_ODELAY, LOG_DAEMON);
       
       if (daemon (0, 0) < 0)
-	error (EXIT_FAILURE, errno, "cannot auto-background");
+	error (EXIT_FAILURE, errno, _("cannot auto-background"));
       forked = 1;
 
       if (write_pid (_PATH_UTMPDPID) < 0)
@@ -235,7 +236,7 @@ make_socket (const char *name)
   /* Create the socket.  */
   sock = socket (PF_UNIX, SOCK_STREAM, 0);
   if (sock < 0)
-    error (EXIT_FAILURE, errno, "cannot create socket");
+    error (EXIT_FAILURE, errno, _("cannot create socket"));
 
   /* Bind a name to the socket.  */
   addr.sun_family = AF_UNIX;
@@ -277,7 +278,7 @@ void handle_requests (void)
       read_fd_set = active_read_fd_set;
       write_fd_set = active_write_fd_set;
       if (select (FD_SETSIZE, &read_fd_set, &write_fd_set, NULL, NULL) < 0)
-	error (EXIT_FAILURE, errno, "cannot get input on sockets");
+	error (EXIT_FAILURE, errno, _("cannot get input on sockets"));
 
       /* Service all the sockets with input pending.  */
       for (fd = 0; fd < FD_SETSIZE; fd++)
@@ -290,7 +291,7 @@ void handle_requests (void)
 
 		  connection = accept_connection (fd, access);
 		  if (connection == NULL)
-		    error (0, errno, "cannot accept connection");
+		    error (0, errno, _("cannot accept connection"));
 
 		  FD_SET (connection->sock, &active_read_fd_set);
 		}
@@ -298,7 +299,7 @@ void handle_requests (void)
 		{
 		  connection = find_connection (fd);
 		  if (connection == NULL)
-		    error (EXIT_FAILURE, 0, "cannot find connection");
+		    error (EXIT_FAILURE, 0, _("cannot find connection"));
 
 		  if (read_data (connection) < 0)
 		    {
@@ -316,7 +317,7 @@ void handle_requests (void)
 	    {
 	      connection = find_connection (fd);
 	      if (connection == NULL)
-		error (EXIT_FAILURE, 0, "cannot find connection");
+		error (EXIT_FAILURE, 0, _("cannot find connection"));
 
 	      if (write_data (connection) < 0)
 		{
diff --git a/login/programs/database.c b/login/programs/database.c
index 3138ae605c..087ec54d26 100644
--- a/login/programs/database.c
+++ b/login/programs/database.c
@@ -42,13 +42,13 @@ static int replace_entry (utmp_database *database, int old_position,
 			  int new_position, const struct utmp *entry);
 static int store_entry (utmp_database *database, int position,
 			const struct utmp *entry);
-static int get_mtime (const char *file, time_t *timer);
+static int get_mtime (int filedes, time_t *timer);
 
 
-/* Open the database specified by FILE and merge it with the
-   contents of the old format file specified by OLD_FILE.  Returns a
-   pointer to a newly allocated structure describing the database, or
-   NULL on error.  */
+/* Open the database specified by FILE and merge it with the contents
+   of the old format file specified by OLD_FILE.  Returns a pointer to
+   a newly allocated structure describing the database, or NULL on
+   error.  */
 utmp_database *
 open_database (const char *file, const char *old_file)
 {
@@ -57,31 +57,54 @@ open_database (const char *file, const char *old_file)
   /* Allocate memory.  */
   database = (utmp_database *) malloc (sizeof (utmp_database));
   if (database == NULL)
-    return NULL;
+    {
+      error (0, 0, _("memory exhausted"));
+      return NULL;
+    }
 
   memset (database, 0, sizeof (utmp_database));
 
-  /* Open database.  */
-  database->fd = open (file, O_RDWR);
+  /* Open database, create it if it doesn't exist already.  */
+  database->fd = open (file, O_RDWR | O_CREAT);
   if (database->fd < 0)
-    goto fail;
+    {
+      error (0, errno, "%s", file);
+      goto return_error;
+    }
 
-  database->old_fd = open (old_file, O_RDWR);
-  if (database->old_fd < 0)
-    goto fail;
+  database->file = strdup (file);
+  if (database->file == NULL)
+    {
+      error (0, 0, _("memory exhausted"));
+      goto return_error;
+    }
+
+  if (old_file)
+    {
+      database->old_fd = open (old_file, O_RDWR);
+      if (database->old_fd < 0)
+	{
+	  error (0, errno, "%s", old_file);
+	  goto return_error;
+	}
   
-  if ((file && !(database->file = strdup (file)))
-      || (old_file && !(database->old_file = strdup (old_file))))
-    goto fail;
+      database->old_file = strdup (old_file);
+      if (database->old_file == NULL)
+	{
+	  error (0, 0, _("memory exhausted"));
+	  goto return_error;
+	}
+    }
 
-  if (initialize_database (database) < 0
-      || synchronize_database (database) < 0)
-    goto fail;
+  /* Initialize database.  */
+  if (initialize_database (database) < 0)
+    goto return_error;
   
   return database;
 
-fail:
+return_error:
   close_database (database);
+  
   return NULL;
 }
 
@@ -100,8 +123,12 @@ synchronize_database (utmp_database *database)
       
       curtime = time (NULL);
       
-      if (get_mtime (database->old_file, &mtime) < 0)
-	return -1;
+      if (get_mtime (database->old_fd, &mtime) < 0)
+	{
+	  error (0, errno, _("%s: cannot get modification time"),
+		 database->old_file);
+	  return -1;
+	}
       
       if (mtime >= database->mtime)
 	{
@@ -118,7 +145,10 @@ synchronize_database (utmp_database *database)
 		  || !compare_entry (&old_entry, &entry))
 		{
 		  if (write_entry (database, position, &old_entry) < 0)
-		    return -1;
+		    {
+		      error (0, errno, "%s", database->file);
+		      return -1;
+		    }
 		}
 
 	      position++;
@@ -325,13 +355,19 @@ initialize_database (utmp_database *database)
 	      || entry.ut_type == OLD_TIME || entry.ut_type == NEW_TIME)
 	    {
 	      if (store_state_entry (database, position, &entry) < 0)
-		return -1;
+		{
+		  error (0, errno, "%s", database->file);
+		  return -1;
+		}
 	    }
 	  else
 #endif
 	    {
 	      if (store_process_entry (database, position, &entry) < 0)
-		return -1;
+		{
+		  error (0, errno, "%s", database->file);
+		  return -1;
+		}
 	    }
 
 	  /* Update position.  */
@@ -344,14 +380,17 @@ initialize_database (utmp_database *database)
 	    break;
 
 	  if (write_old_entry (database, position, &entry) < 0)
-	    return -1;
+	    {
+	      error (0, errno, "%s", database->file);
+	      return -1;
+	    }
 
 	  /* Update position.  */
 	  position++;
 	}
     }
 
-  return 0;
+  return synchronize_database (database);
 }
 
 
@@ -472,14 +511,14 @@ store_entry (utmp_database *database, int position,
 }
 
 
-/* Get modification time of FILE and put it in TIMER.  returns 0 if
-   successful, -1 if not.  */
+/* Get modification time of the file with file descriptor FILEDES and
+   put it in TIMER.  Returns 0 if successful, -1 if not.  */
 static int
-get_mtime (const char *file, time_t *timer)
+get_mtime (int filedes, time_t *timer)
 {
   struct stat st;
   
-  if (stat (file, &st) < 0)
+  if (fstat (filedes, &st) < 0)
     return -1;
 
   *timer = st.st_mtime;